@@ -85,6 +85,7 @@ CLI *alloc_client_session(SERVICE_OPTIONS *opt, SOCKET rfd, SOCKET wfd) {
     c->local_rfd.fd=rfd;
     c->local_wfd.fd=wfd;
     c->seq=seq++;
+    c->opt->seq++;
     return c;
 }
 
@@ -254,7 +255,8 @@ NOEXPORT void client_run(CLI *c) {
 
         /* free the client context */
     str_free(c->connect_addr.addr);
-    /* we currently don't make a local copy of c->connect_addr.session */
+    /* a client does not have its own local copy of
+       c->connect_addr.session and c->connect_addr.fd */
     s_poll_free(c->fds);
     c->fds=NULL;
 }
@@ -646,21 +648,18 @@ NOEXPORT void transfer(CLI *c) {
                     s_poll_err(c->fds, c->sock_rfd->fd)) {
                 err=get_socket_error(c->sock_rfd->fd);
                 if(err)
-                    s_log(LOG_ERR, "socket fd: %s (%d)",
-                        s_strerror(err), err);
+                    log_error(LOG_INFO, err, "socket fd");
             }
         } else {
             if(sock_can_rd && s_poll_err(c->fds, c->sock_rfd->fd)) {
                 err=get_socket_error(c->sock_rfd->fd);
                 if(err)
-                    s_log(LOG_ERR, "socket rfd: %s (%d)",
-                        s_strerror(err), err);
+                    log_error(LOG_INFO, err, "socket rfd");
             }
             if(sock_can_wr && s_poll_err(c->fds, c->sock_wfd->fd)) {
                 err=get_socket_error(c->sock_wfd->fd);
                 if(err)
-                    s_log(LOG_ERR, "socket wfd: %s (%d)",
-                        s_strerror(err), err);
+                    log_error(LOG_INFO, err, "socket wfd");
             }
         }
         if(c->ssl_rfd->fd==c->ssl_wfd->fd) {
@@ -668,22 +667,19 @@ NOEXPORT void transfer(CLI *c) {
                     s_poll_err(c->fds, c->ssl_rfd->fd)) {
                 err=get_socket_error(c->ssl_rfd->fd);
                 if(err)
-                    s_log(LOG_ERR, "TLS fd: %s (%d)",
-                        s_strerror(err), err);
+                    log_error(LOG_INFO, err, "TLS fd");
             }
         } else {
             if(ssl_can_rd && s_poll_err(c->fds, c->ssl_rfd->fd)) {
                 err=get_socket_error(c->ssl_rfd->fd);
                 if(err)
-                    s_log(LOG_ERR, "TLS rfd: %s (%d)",
-                        s_strerror(err), err);
+                    log_error(LOG_INFO, err, "TLS rfd");
             }
             if(c->ssl_rfd->fd!=c->ssl_wfd->fd &&
                     ssl_can_wr && s_poll_err(c->fds, c->ssl_wfd->fd)) {
                 err=get_socket_error(c->ssl_wfd->fd);
                 if(err)
-                    s_log(LOG_ERR, "TLS wfd: %s (%d)",
-                        s_strerror(err), err);
+                    log_error(LOG_INFO, err, "TLS wfd");
             }
         }
 
@@ -736,6 +732,10 @@ NOEXPORT void transfer(CLI *c) {
                 SSL_set_shutdown(c->ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
                 shutdown_wants_read=shutdown_wants_write=0;
                 break;
+            case SSL_ERROR_ZERO_RETURN: /* connection closed */
+                SSL_set_shutdown(c->ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+                shutdown_wants_read=shutdown_wants_write=0;
+                break;
             case SSL_ERROR_WANT_WRITE:
                 s_log(LOG_DEBUG, "SSL_shutdown returned WANT_WRITE: retrying");
                 shutdown_wants_read=0;
@@ -1109,6 +1109,7 @@ NOEXPORT void auth_user(CLI *c, char *accepted_address) {
 #endif
     SOCKADDR_UNION ident;     /* IDENT socket name */
     char *line, *type, *system, *user;
+    unsigned remote_port, local_port;
 
     if(!c->opt->username)
         return; /* -u option not specified */
@@ -1136,9 +1137,10 @@ NOEXPORT void auth_user(CLI *c, char *accepted_address) {
     if(s_connect(c, &ident, addr_len(&ident)))
         longjmp(c->err, 1);
     s_log(LOG_DEBUG, "IDENT server connected");
-    fd_printf(c, c->fd, "%u , %u",
-        ntohs(c->peer_addr.in.sin_port),
-        ntohs(c->opt->local_addr.in.sin_port));
+    remote_port=ntohs(c->peer_addr.in.sin_port);
+    local_port=(unsigned)(c->opt->local_addr.addr ?
+        ntohs(c->opt->local_addr.addr[0].in.sin_port) : 0);
+    fd_printf(c, c->fd, "%u , %u", remote_port, local_port);
     line=fd_getline(c, c->fd);
     closesocket(c->fd);
     c->fd=INVALID_SOCKET; /* avoid double close on cleanup */
@@ -1443,9 +1445,7 @@ NOEXPORT unsigned idx_cache_retrieve(CLI *c) {
     }
 
     if(c->opt->failover==FAILOVER_RR) {
-        /* the race condition here can be safely ignored */
-        i=c->connect_addr.parent->rr;
-        c->connect_addr.parent->rr=(i+1)%c->connect_addr.num;
+        i=(c->connect_addr.start+c->opt->seq)%c->connect_addr.num;
         s_log(LOG_INFO, "failover: round-robin, starting at entry #%d", i);
     } else {
         i=0;
