@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -39,14 +39,14 @@
 #include "prototypes.h"
 
 #ifdef USE_PTHREAD
-void *cron_thread(void *arg);
+NOEXPORT void *cron_thread(void *arg);
 #endif
 #ifdef USE_WIN32
-void cron_thread(void *arg);
+NOEXPORT void cron_thread(void *arg);
 #endif
 #if defined(USE_PTHREAD) || defined(USE_WIN32)
-void cron_worker(void);
-void cron_dhparam(void);
+NOEXPORT void cron_worker(void);
+NOEXPORT void cron_dh_param(void);
 #endif
 
 #if defined(USE_PTHREAD)
@@ -73,12 +73,12 @@ int cron_init() {
     return 0;
 }
 
-void *cron_thread(void *arg) {
+NOEXPORT void *cron_thread(void *arg) {
 #ifdef SCHED_BATCH
     struct sched_param param;
 #endif
 
-    (void)arg; /* skip warning about unused parameter */
+    (void)arg; /* squash the unused parameter warning */
     tls_alloc(NULL, NULL, "cron");
 #ifdef SCHED_BATCH
     param.sched_priority=0;
@@ -97,10 +97,10 @@ int cron_init() {
     return 0;
 }
 
-void cron_thread(void *arg) {
-    (void)arg; /* skip warning about unused parameter */
+NOEXPORT void cron_thread(void *arg) {
+    (void)arg; /* squash the unused parameter warning */
     tls_alloc(NULL, NULL, "cron");
-    if(!SetThreadPriority(GetCurrentThread(), THREAD_MODE_BACKGROUND_BEGIN))
+    if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_LOWEST))
         ioerror("SetThreadPriority");
     cron_worker();
     _endthread(); /* it should never be executed */
@@ -118,17 +118,19 @@ int cron_init() {
 /* run the cron job every 24 hours */
 #define CRON_PERIOD (24*60*60)
 
-void cron_worker(void) {
+#if defined(USE_PTHREAD) || defined(USE_WIN32)
+
+NOEXPORT void cron_worker(void) {
     time_t now, then;
     int delay;
 
-    s_log(LOG_DEBUG, "Cron started");
+    s_log(LOG_DEBUG, "Cron thread initialized");
     sleep(60); /* allow the other services to start with idle CPU */
     time(&then);
     for(;;) {
         s_log(LOG_INFO, "Executing cron jobs");
 #ifndef OPENSSL_NO_DH
-        cron_dhparam();
+        cron_dh_param();
 #endif /* OPENSSL_NO_DH */
         time(&now);
         s_log(LOG_INFO, "Cron jobs completed in %d seconds", (int)(now-then));
@@ -141,20 +143,25 @@ void cron_worker(void) {
             then=now+delay;
         }
         s_log(LOG_DEBUG, "Waiting %d seconds", delay);
-        sleep((unsigned)delay);
+        do { /* retry sleep() if it was interrupted by a signal */
+            sleep((unsigned)delay);
+            time(&now);
+            delay=(int)(then-now);
+        } while(delay>0);
         s_log(LOG_INFO, "Reopening log file");
         signal_post(SIGNAL_REOPEN_LOG);
     }
 }
 
 #ifndef OPENSSL_NO_DH
-void cron_dhparam(void) {
+NOEXPORT void cron_dh_param(void) {
     SERVICE_OPTIONS *opt;
     DH *dh;
 
     if(!dh_needed)
         return;
 
+    s_log(LOG_NOTICE, "Updating DH parameters");
 #if OPENSSL_VERSION_NUMBER>=0x0090800fL
     /* generate 2048-bit DH parameters */
     dh=DH_new();
@@ -175,13 +182,20 @@ void cron_dhparam(void) {
     }
 #endif
 
+    /* update global dh_params for future configuration reloads */
+    stunnel_write_lock(&stunnel_locks[LOCK_DH]);
+    DH_free(dh_params);
+    dh_params=dh;
+    stunnel_write_unlock(&stunnel_locks[LOCK_DH]);
+
     /* set for all sections that require it */
     for(opt=service_options.next; opt; opt=opt->next)
         if(opt->option.dh_needed)
             SSL_CTX_set_tmp_dh(opt->ctx, dh);
-    DH_free(dh);
     s_log(LOG_NOTICE, "DH parameters updated");
 }
 #endif /* OPENSSL_NO_DH */
 
+#endif /* USE_PTHREAD || USE_WIN32 */
+
 /* end of cron.c */
