@@ -1,6 +1,6 @@
 /*
- *   stunnel       Universal SSL tunnel
- *   Copyright (C) 1998-2014 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   stunnel       TLS offloading and load-balancing proxy
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -116,7 +116,7 @@ DISK_FILE *file_open(char *name, FILE_MODE mode) {
     flags|=O_CLOEXEC;
 #endif /* O_CLOEXEC */
     fd=open(name, flags, 0640);
-    if(fd<0)
+    if(fd==INVALID_SOCKET)
         return NULL;
 
     /* setup df structure */
@@ -139,14 +139,14 @@ void file_close(DISK_FILE *df) {
     str_free(df);
 }
 
-int file_getline(DISK_FILE *df, char *line, int len) {
+ssize_t file_getline(DISK_FILE *df, char *line, int len) {
     /* this version is really slow, but performance is not important here */
     /* (no buffering is implemented) */
-    int i;
+    ssize_t i;
 #ifdef USE_WIN32
     DWORD num;
 #else /* USE_WIN32 */
-    int num;
+    ssize_t num;
 #endif /* USE_WIN32 */
 
     if(!df) /* not opened */
@@ -173,13 +173,13 @@ int file_getline(DISK_FILE *df, char *line, int len) {
     return i;
 }
 
-int file_putline(DISK_FILE *df, char *line) {
-    int len;
+ssize_t file_putline(DISK_FILE *df, char *line) {
     char *buff;
+    size_t len;
 #ifdef USE_WIN32
     DWORD num;
 #else /* USE_WIN32 */
-    int num;
+    ssize_t num;
 #endif /* USE_WIN32 */
 
     len=strlen(line);
@@ -190,57 +190,73 @@ int file_putline(DISK_FILE *df, char *line) {
 #endif /* USE_WIN32 */
     buff[len++]='\n'; /* LF */
 #ifdef USE_WIN32
-    WriteFile(df->fh, buff, len, &num, NULL);
+    WriteFile(df->fh, buff, (DWORD)len, &num, NULL);
 #else /* USE_WIN32 */
     /* no file -> write to stderr */
     num=write(df ? df->fd : 2, buff, len);
 #endif /* USE_WIN32 */
     str_free(buff);
-    return num;
+    return (ssize_t)num;
+}
+
+int file_permissions(const char *file_name) {
+#if !defined(USE_WIN32) && !defined(USE_OS2)
+    struct stat sb; /* buffer for stat */
+
+    /* check permissions of the private key file */
+    if(stat(file_name, &sb)) {
+        ioerror(file_name);
+        return 1; /* FAILED */
+    }
+    if(sb.st_mode & 7)
+        s_log(LOG_WARNING,
+            "Insecure file permissions on %s", file_name);
+#else
+    (void)file_name; /* squash the unused parameter warning */
+#endif
+    return 0;
 }
 
 #ifdef USE_WIN32
 
 LPTSTR str2tstr(LPCSTR in) {
     LPTSTR out;
+#ifdef UNICODE
     int len;
 
-#ifdef UNICODE
     len=MultiByteToWideChar(CP_UTF8, 0, in, -1, NULL, 0);
     if(!len)
         return str_tprintf(TEXT("MultiByteToWideChar() failed"));
-    out=str_alloc((len+1)*sizeof(WCHAR));
+    out=str_alloc(((size_t)len+1)*sizeof(WCHAR));
     len=MultiByteToWideChar(CP_UTF8, 0, in, -1, out, len);
     if(!len) {
         str_free(out);
         return str_tprintf(TEXT("MultiByteToWideChar() failed"));
     }
 #else
-    len=strlen(in);
-    out=str_alloc(len+1);
-    strcpy(out, in);
+    /* FIXME: convert UTF-8 to native codepage */
+    out=str_dup(in);
 #endif
     return out;
 }
 
 LPSTR tstr2str(LPCTSTR in) {
     LPSTR out;
+#ifdef UNICODE
     int len;
 
-#ifdef UNICODE
     len=WideCharToMultiByte(CP_UTF8, 0, in, -1, NULL, 0, NULL, NULL);
     if(!len)
         return str_printf("WideCharToMultiByte() failed");
-    out=str_alloc(len+1);
+    out=str_alloc((size_t)len+1);
     len=WideCharToMultiByte(CP_UTF8, 0, in, -1, out, len, NULL, NULL);
     if(!len) {
         str_free(out);
         return str_printf("WideCharToMultiByte() failed");
     }
 #else
-    len=strlen(in);
-    out=str_alloc(len+1);
-    strcpy(out, in);
+    /* FIXME: convert native codepage to UTF-8 */
+    out=str_dup(in);
 #endif
     return out;
 }
