@@ -38,8 +38,8 @@
 #include "common.h"
 #include "prototypes.h"
 
-NOEXPORT void log_raw(const SERVICE_OPTIONS *, const int,
-    const char *, const char *, const char *);
+NOEXPORT void log_queue(SERVICE_OPTIONS *, int, char *, char *, char *);
+NOEXPORT void log_raw(SERVICE_OPTIONS *, int, char *, char *, char *);
 NOEXPORT void safestring(char *);
 
 static DISK_FILE *outfile=NULL;
@@ -115,34 +115,9 @@ void log_close(void) {
     stunnel_write_unlock(&stunnel_locks[LOCK_LOG_MODE]);
 }
 
-void log_flush(LOG_MODE new_mode) {
-    struct LIST *tmp;
-
-    stunnel_write_lock(&stunnel_locks[LOCK_LOG_MODE]);
-    /* prevent changing LOG_MODE_CONFIGURED to LOG_MODE_ERROR
-     * once stderr file descriptor is closed */
-    if(log_mode!=LOG_MODE_CONFIGURED)
-        log_mode=new_mode;
-    /* log_raw() will use the new value of log_mode */
-    stunnel_write_lock(&stunnel_locks[LOCK_LOG_BUFFER]);
-    while(head) {
-        log_raw(head->opt, head->level, head->stamp, head->id, head->text);
-        str_free(head->stamp);
-        str_free(head->id);
-        str_free(head->text);
-        tmp=head;
-        head=head->next;
-        str_free(tmp);
-    }
-    head=tail=NULL;
-    stunnel_write_unlock(&stunnel_locks[LOCK_LOG_BUFFER]);
-    stunnel_write_unlock(&stunnel_locks[LOCK_LOG_MODE]);
-}
-
 void s_log(int level, const char *format, ...) {
     va_list ap;
     char *text, *stamp, *id;
-    struct LIST *tmp;
 #ifdef USE_WIN32
     DWORD libc_error;
 #else
@@ -156,6 +131,9 @@ void s_log(int level, const char *format, ...) {
 #endif
     TLS_DATA *tls_data;
 
+    libc_error=get_last_error();
+    socket_error=get_last_socket_error();
+
     tls_data=tls_get();
     if(!tls_data) {
         tls_data=tls_alloc(NULL, NULL, "log");
@@ -164,68 +142,93 @@ void s_log(int level, const char *format, ...) {
     }
 
     /* performance optimization: skip the trivial case early */
-    if(log_mode==LOG_MODE_CONFIGURED && level>tls_data->opt->log_level)
-        return;
-
-    libc_error=get_last_error();
-    socket_error=get_last_socket_error();
-
-    /* format the id to be logged */
-    time(&gmt);
+    if(log_mode!=LOG_MODE_CONFIGURED || level<=tls_data->opt->log_level) {
+        /* format the id to be logged */
+        time(&gmt);
 #if defined(HAVE_LOCALTIME_R) && defined(_REENTRANT)
-    timeptr=localtime_r(&gmt, &timestruct);
+        timeptr=localtime_r(&gmt, &timestruct);
 #else
-    timeptr=localtime(&gmt);
+        timeptr=localtime(&gmt);
 #endif
-    stamp=str_printf("%04d.%02d.%02d %02d:%02d:%02d",
-        timeptr->tm_year+1900, timeptr->tm_mon+1, timeptr->tm_mday,
-        timeptr->tm_hour, timeptr->tm_min, timeptr->tm_sec);
-    id=str_printf("LOG%d[%s]", level, tls_data->id);
+        stamp=str_printf("%04d.%02d.%02d %02d:%02d:%02d",
+            timeptr->tm_year+1900, timeptr->tm_mon+1, timeptr->tm_mday,
+            timeptr->tm_hour, timeptr->tm_min, timeptr->tm_sec);
+        id=str_printf("LOG%d[%s]", level, tls_data->id);
 
-    /* format the text to be logged */
-    va_start(ap, format);
-    text=str_vprintf(format, ap);
-    va_end(ap);
-    safestring(text);
+        /* format the text to be logged */
+        va_start(ap, format);
+        text=str_vprintf(format, ap);
+        va_end(ap);
+        safestring(text);
 
-    stunnel_read_lock(&stunnel_locks[LOCK_LOG_MODE]);
-    if(log_mode==LOG_MODE_BUFFER) { /* save the text to log it later */
-        stunnel_write_lock(&stunnel_locks[LOCK_LOG_BUFFER]);
-        tmp=str_alloc_detached(sizeof(struct LIST));
-        tmp->next=NULL;
-        tmp->opt=tls_data->opt;
-        tmp->level=level;
-        tmp->stamp=stamp;
-        str_detach(tmp->stamp);
-        tmp->id=id;
-        str_detach(tmp->id);
-        tmp->text=text;
-        str_detach(tmp->text);
-        if(tail)
-            tail->next=tmp;
+        /* either log or queue for logging */
+        stunnel_read_lock(&stunnel_locks[LOCK_LOG_MODE]);
+        if(log_mode==LOG_MODE_BUFFER)
+            log_queue(tls_data->opt, level, stamp, id, text);
         else
-            head=tmp;
-        tail=tmp;
-        stunnel_write_unlock(&stunnel_locks[LOCK_LOG_BUFFER]);
-    } else { /* ready log the text directly */
-        log_raw(tls_data->opt, level, stamp, id, text);
-        str_free(stamp);
-        str_free(id);
-        str_free(text);
+            log_raw(tls_data->opt, level, stamp, id, text);
+        stunnel_read_unlock(&stunnel_locks[LOCK_LOG_MODE]);
     }
-    stunnel_read_unlock(&stunnel_locks[LOCK_LOG_MODE]);
 
     set_last_error(libc_error);
     set_last_socket_error(socket_error);
 }
 
-NOEXPORT void log_raw(const SERVICE_OPTIONS *opt,
-        const int level, const char *stamp,
-        const char *id, const char *text) {
+NOEXPORT void log_queue(SERVICE_OPTIONS *opt,
+        int level, char *stamp, char *id, char *text) {
+    struct LIST *tmp;
+
+    /* make a new element */
+    tmp=str_alloc_detached(sizeof(struct LIST));
+    tmp->next=NULL;
+    tmp->opt=opt;
+    tmp->level=level;
+    tmp->stamp=stamp;
+    str_detach(tmp->stamp);
+    tmp->id=id;
+    str_detach(tmp->id);
+    tmp->text=text;
+    str_detach(tmp->text);
+
+    /* append the new element to the list */
+    stunnel_write_lock(&stunnel_locks[LOCK_LOG_BUFFER]);
+    if(tail)
+        tail->next=tmp;
+    else
+        head=tmp;
+    tail=tmp;
+    stunnel_write_unlock(&stunnel_locks[LOCK_LOG_BUFFER]);
+}
+
+void log_flush(LOG_MODE new_mode) {
+    stunnel_write_lock(&stunnel_locks[LOCK_LOG_MODE]);
+    /* prevent changing LOG_MODE_CONFIGURED to LOG_MODE_ERROR
+     * once stderr file descriptor is closed */
+    if(log_mode!=LOG_MODE_CONFIGURED)
+        log_mode=new_mode;
+    /* log_raw() will use the new value of log_mode */
+    stunnel_write_lock(&stunnel_locks[LOCK_LOG_BUFFER]);
+    while(head) {
+        struct LIST *tmp=head;
+        head=head->next;
+        log_raw(tmp->opt, tmp->level, tmp->stamp, tmp->id, tmp->text);
+        str_free(tmp);
+    }
+    head=tail=NULL;
+    stunnel_write_unlock(&stunnel_locks[LOCK_LOG_BUFFER]);
+    stunnel_write_unlock(&stunnel_locks[LOCK_LOG_MODE]);
+}
+
+NOEXPORT void log_raw(SERVICE_OPTIONS *opt,
+        int level, char *stamp, char *id, char *text) {
     char *line;
 
-    /* build the line and log it to syslog/file */
-    if(log_mode==LOG_MODE_CONFIGURED) { /* configured */
+    /* NOTE: opt->log_level may have changed since s_log().
+     * It is important to use the new value and not the old one. */
+
+    /* build the line and log it to syslog/file if configured */
+    switch(log_mode) {
+    case LOG_MODE_CONFIGURED:
         line=str_printf("%s %s: %s", stamp, id, text);
         if(level<=opt->log_level) {
 #if !defined(USE_WIN32) && !defined(__vms)
@@ -233,15 +236,25 @@ NOEXPORT void log_raw(const SERVICE_OPTIONS *opt,
                 syslog(level, "%s: %s", id, text);
 #endif /* USE_WIN32, __vms */
             if(outfile)
-                file_putline(outfile, line); /* send log to file */
+                file_putline(outfile, line);
         }
-    } else if(log_mode==LOG_MODE_ERROR) {
+        break;
+    case LOG_MODE_ERROR:
+        /* don't log the id or the time stamp */
         if(level>=0 && level<=7) /* just in case */
             line=str_printf("[%c] %s", "***!:.  "[level], text);
         else
             line=str_printf("[?] %s", text);
-    } else /* LOG_MODE_INFO */
-        line=str_dup(text); /* don't log the time stamp in error mode */
+        break;
+    default: /* LOG_MODE_INFO */
+        /* don't log the level, the id or the time stamp */
+        line=str_dup(text);
+    }
+
+    /* free the memory */
+    str_free(stamp);
+    str_free(id);
+    str_free(text);
 
     /* log the line to the UI (GUI, stderr, etc.) */
     if(log_mode==LOG_MODE_ERROR ||
