@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -46,10 +46,6 @@ typedef struct tls_data_struct TLS_DATA;
 
 /**************************************** data structures */
 
-/* non-zero constants for the "redirect" option */
-#define REDIRECT_ON         1
-#define REDIRECT_OFF        2
-
 #if defined (USE_WIN32)
 #define ICON_IMAGE HICON
 #elif defined(__APPLE__)
@@ -102,8 +98,9 @@ typedef struct name_list_struct {
 typedef struct sockaddr_list {                          /* list of addresses */
     struct sockaddr_list *parent;   /* used by copies to locate their parent */
     SOCKADDR_UNION *addr;                     /* array of resolved addresses */
+    SOCKET *fd;                       /* array of accepting file descriptors */
     SSL_SESSION **session;                /* array of cached client sessions */
-    unsigned rr;                          /* current address for round-robin */
+    unsigned start;              /* initial address for round-robin failover */
     unsigned num;                             /* how many addresses are used */
     int passive;                                         /* listening socket */
     NAME_LIST *names;                          /* a list of unresolved names */
@@ -129,7 +126,6 @@ typedef struct {
 #ifdef HAVE_CHROOT
     char *chroot_dir;
 #endif
-    unsigned long dpid;
     char *pidfile;
 #endif
 
@@ -242,7 +238,6 @@ typedef struct service_options_struct {
 #endif /* !defined(OPENSSL_NO_ENGINE) */
 
         /* service-specific data for client.c */
-    SOCKET fd;     /* file descriptor accepting connections for this service */
     SSL_SESSION *session;                           /* recently used session */
     char *exec_name;                          /* program name for local mode */
 #ifdef USE_WIN32
@@ -250,13 +245,14 @@ typedef struct service_options_struct {
 #else
     char **exec_args;                    /* program arguments for local mode */
 #endif
-    SOCKADDR_UNION local_addr, source_addr;
-    SOCKADDR_LIST connect_addr, redirect_addr;
+    SOCKADDR_UNION source_addr;
+    SOCKADDR_LIST local_addr, connect_addr, redirect_addr;
     int timeout_busy;                       /* maximum waiting for data time */
     int timeout_close;                          /* maximum close_notify time */
     int timeout_connect;                           /* maximum connect() time */
     int timeout_idle;                        /* maximum idle connection time */
     enum {FAILOVER_RR, FAILOVER_PRIO} failover;         /* failover strategy */
+    unsigned seq;              /* sequential number for round-robin failover */
     char *username;
 
         /* service-specific data for protocol.c */
@@ -410,7 +406,6 @@ typedef struct {
     FD *ssl_rfd, *ssl_wfd; /* read and write TLS descriptors */
     uint64_t sock_bytes, ssl_bytes; /* bytes written to socket and TLS */
     s_poll_set *fds; /* file descriptors */
-    uintptr_t redirect; /* redirect to another destination after failed auth */
 } CLI;
 
 /**************************************** prototypes for stunnel.c */
@@ -491,17 +486,15 @@ int cron_init(void);
 
 /**************************************** prototypes for ssl.c */
 
-extern int index_cli, index_opt, index_redirect, index_addr;
+extern int index_ssl_cli, index_ssl_ctx_opt;
+extern int index_session_authenticated, index_session_connect_address;
 
 int ssl_init(void);
 int ssl_configure(GLOBAL_OPTIONS *);
 
 /**************************************** prototypes for ctx.c */
 
-typedef struct {
-    SERVICE_OPTIONS *section;
-    char pass[PEM_BUFSIZE];
-} UI_DATA;
+extern SERVICE_OPTIONS *current_section;
 
 #ifndef OPENSSL_NO_DH
 extern DH *dh_params;
@@ -640,15 +633,20 @@ int getnameinfo(const struct sockaddr *, socklen_t,
 
 /**************************************** prototypes for sthreads.c */
 
-#ifdef USE_FORK
+#if defined(USE_PTHREAD) || defined(USE_WIN32)
 
-#define CRYPTO_THREAD_read_lock(type) {}
-#define CRYPTO_THREAD_read_unlock(type) {}
-#define CRYPTO_THREAD_write_lock(type) {}
-#define CRYPTO_THREAD_write_unlock(type) {}
-#define CRYPTO_atomic_add(addr,amount,result,type) (*addr+=amount)
-
-#else /* USE_FORK */
+struct CRYPTO_dynlock_value {
+#ifdef USE_PTHREAD
+    pthread_rwlock_t rwlock;
+#endif
+#ifdef USE_WIN32
+    CRITICAL_SECTION critical_section;
+#endif
+    const char *init_file, *read_lock_file, *write_lock_file,
+        *read_unlock_file, *write_unlock_file, *destroy_file;
+    int init_line, read_lock_line, write_lock_line,
+        read_unlock_line, write_unlock_line, destroy_line;
+};
 
 typedef enum {
     LOCK_SESSION, LOCK_ADDR,
@@ -664,30 +662,37 @@ typedef enum {
 #endif /* OPENSSL_NO_DH */
     STUNNEL_LOCKS                           /* number of locks */
 } LOCK_TYPE;
-#if OPENSSL_VERSION_NUMBER < 0x10100004L
-typedef int STUNNEL_RWLOCK;
-#else
-typedef CRYPTO_RWLOCK *STUNNEL_RWLOCK;
-#endif
-extern STUNNEL_RWLOCK stunnel_locks[STUNNEL_LOCKS];
-#if OPENSSL_VERSION_NUMBER>=0x10100004L
-#define CRYPTO_THREAD_read_unlock(type) CRYPTO_THREAD_unlock(type)
-#define CRYPTO_THREAD_write_unlock(type) CRYPTO_THREAD_unlock(type)
-#else
-/* Emulate the OpenSSL 1.1 locking API for older OpenSSL versions */
-#define CRYPTO_THREAD_read_lock(type) \
-    if(type) CRYPTO_lock(CRYPTO_LOCK|CRYPTO_READ,type,__FILE__,__LINE__)
-#define CRYPTO_THREAD_read_unlock(type) \
-    if(type) CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_READ,type,__FILE__,__LINE__)
-#define CRYPTO_THREAD_write_lock(type) \
-    if(type) CRYPTO_lock(CRYPTO_LOCK|CRYPTO_WRITE,type,__FILE__,__LINE__)
-#define CRYPTO_THREAD_write_unlock(type) \
-    if(type) CRYPTO_lock(CRYPTO_UNLOCK|CRYPTO_WRITE,type,__FILE__,__LINE__)
+extern struct CRYPTO_dynlock_value stunnel_locks[STUNNEL_LOCKS];
+
+void stunnel_rwlock_init_debug(struct CRYPTO_dynlock_value *, const char *, int);
+void stunnel_read_lock_debug(struct CRYPTO_dynlock_value *, const char *, int);
+void stunnel_write_lock_debug(struct CRYPTO_dynlock_value *, const char *, int);
+void stunnel_read_unlock_debug(struct CRYPTO_dynlock_value *, const char *, int);
+void stunnel_write_unlock_debug(struct CRYPTO_dynlock_value *, const char *, int);
+void stunnel_rwlock_destroy_debug(struct CRYPTO_dynlock_value *, const char *, int);
+
+#define stunnel_rwlock_init(x) stunnel_rwlock_init_debug((x),__FILE__,__LINE__)
+#define stunnel_read_lock(x) stunnel_read_lock_debug((x),__FILE__,__LINE__)
+#define stunnel_write_lock(x) stunnel_write_lock_debug((x),__FILE__,__LINE__)
+#define stunnel_read_unlock(x) stunnel_read_unlock_debug((x),__FILE__,__LINE__)
+#define stunnel_write_unlock(x) stunnel_write_unlock_debug((x),__FILE__,__LINE__)
+#define stunnel_rwlock_destroy(x) stunnel_rwlock_destroy_debug((x),__FILE__,__LINE__)
+
+#if OPENSSL_VERSION_NUMBER<0x10100004L
 #define CRYPTO_atomic_add(addr,amount,result,type) \
     *result = type ? CRYPTO_add(addr,amount,type) : (*addr+=amount)
 #endif
 
-#endif /* USE_FORK */
+#else /* defined(USE_PTHREAD) || defined(USE_WIN32) */
+
+#define stunnel_rwlock_init(x) {}
+#define stunnel_read_lock(x) {}
+#define stunnel_write_lock(x) {}
+#define stunnel_read_unlock(x) {}
+#define stunnel_write_unlock(x) {}
+#define stunnel_rwlock_destroy(x) {}
+
+#endif /* defined(USE_PTHREAD) || defined(USE_WIN32) */
 
 int sthreads_init(void);
 unsigned long stunnel_process_id(void);
@@ -802,7 +807,7 @@ void ui_new_log(const char *);
 void message_box(LPCTSTR, const UINT);
 #endif /* USE_WIN32 */
 
-int passwd_cb(char *, int, int, void *);
+int ui_passwd_cb(char *, int, int, void *);
 #ifndef OPENSSL_NO_ENGINE
 UI_METHOD *UI_stunnel(void);
 #endif /* !defined(OPENSSL_NO_ENGINE) */
