@@ -138,6 +138,7 @@ unsigned name2addr(SOCKADDR_UNION *addr, char *name, int passive) {
     if(retval)
         addrlist2addr(addr, addr_list);
     str_free(addr_list->addr);
+    str_free(addr_list->fd);
     str_free(addr_list->session);
     str_free(addr_list);
     return retval;
@@ -154,6 +155,7 @@ unsigned hostport2addr(SOCKADDR_UNION *addr,
     if(num)
         addrlist2addr(addr, addr_list);
     str_free(addr_list->addr);
+    str_free(addr_list->fd);
     str_free(addr_list->session);
     str_free(addr_list);
     return num;
@@ -196,6 +198,9 @@ unsigned name2addrlist(SOCKADDR_LIST *addr_list, char *name) {
             (addr_list->num+1)*sizeof(SOCKADDR_UNION));
         addr_list->addr[addr_list->num].un.sun_family=AF_UNIX;
         strcpy(addr_list->addr[addr_list->num].un.sun_path, name);
+        addr_list->fd=str_realloc(addr_list->fd,
+            (addr_list->num+1)*sizeof(SOCKET));
+        addr_list->fd[addr_list->num]=INVALID_SOCKET;
         addr_list->session=str_realloc(addr_list->session,
             (addr_list->num+1)*sizeof(SSL_SESSION *));
         addr_list->session[addr_list->num]=NULL;
@@ -291,6 +296,9 @@ unsigned hostport2addrlist(SOCKADDR_LIST *addr_list,
             (addr_list->num+1)*sizeof(SOCKADDR_UNION));
         memcpy(&addr_list->addr[addr_list->num], cur->ai_addr,
             (size_t)cur->ai_addrlen);
+        addr_list->fd=str_realloc(addr_list->fd,
+            (addr_list->num+1)*sizeof(SOCKET));
+        addr_list->fd[addr_list->num]=INVALID_SOCKET;
         addr_list->session=str_realloc(addr_list->session,
             (addr_list->num+1)*sizeof(SSL_SESSION *));
         addr_list->session[addr_list->num]=NULL;
@@ -312,8 +320,9 @@ void addrlist_clear(SOCKADDR_LIST *addr_list, int passive) {
 NOEXPORT void addrlist_reset(SOCKADDR_LIST *addr_list) {
     addr_list->num=0;
     addr_list->addr=NULL;
+    addr_list->fd=NULL;
     addr_list->session=NULL;
-    addr_list->rr=0; /* reset the round-robin counter */
+    addr_list->start=0;
     addr_list->parent=addr_list; /* allow a copy to locate its parent */
 }
 
@@ -325,7 +334,8 @@ unsigned addrlist_dup(SOCKADDR_LIST *dst, const SOCKADDR_LIST *src) {
     } else { /* delayed resolver */
         addrlist_resolve(dst);
     }
-    /* we currently don't make a local copy of src->session */
+    /* a client does not have its own local copy of
+       src->session and src->fd */
     return dst->num;
 }
 
@@ -336,16 +346,13 @@ unsigned addrlist_resolve(SOCKADDR_LIST *addr_list) {
     addrlist_reset(addr_list);
     for(host=addr_list->names; host; host=host->next)
         num+=name2addrlist(addr_list, host->name);
-    switch(num) {
-    case 0:
-    case 1:
-        addr_list->rr=0;
-        break;
-    default:
+    if(num<2) {
+        addr_list->start=0;
+    } else {
         /* randomize the initial value of round-robin counter */
         /* ignore the error value and the distribution bias */
         RAND_bytes((unsigned char *)&rnd, sizeof rnd);
-        addr_list->rr=rnd%num;
+        addr_list->start=rnd%num;
     }
     return num;
 }
