@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@mirt.net>
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -52,11 +52,12 @@ void tls_init() {
     tls_platform_init();
     tls_initialized=1;
     ui_tls=tls_alloc(NULL, NULL, "ui");
-#if OPENSSL_VERSION_NUMBER<0x10100000L
-    /* the ability to set the "memory debug" functions
-     * at runtme was removed in OpenSSL 1.1 */
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
+    CRYPTO_set_mem_functions(str_alloc_detached_debug,
+        str_realloc_detached_debug, str_free_debug);
+#else
     CRYPTO_set_mem_ex_functions(str_alloc_detached_debug,
-        str_realloc_debug, free_function);
+        str_realloc_detached_debug, free_function);
 #endif
 }
 
