@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2016 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2017 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -125,19 +125,13 @@ void ui_new_log(const char *line) {
 
 /**************************************** ctx callbacks */
 
-int passwd_cb(char *buf, int size, int rwflag, void *userdata) {
-    (void)buf; /* squash the unused parameter warning */
-    (void)size; /* squash the unused parameter warning */
-    (void)rwflag; /* squash the unused parameter warning */
-    (void)userdata; /* squash the unused parameter warning */
-    return 0; /* not implemented */
+int ui_passwd_cb(char *buf, int size, int rwflag, void *userdata) {
+    return PEM_def_callback(buf, size, rwflag, userdata);
 }
 
 #ifndef OPENSSL_NO_ENGINE
-int pin_cb(UI *ui, UI_STRING *uis) {
-    (void)ui; /* squash the unused parameter warning */
-    (void)uis; /* squash the unused parameter warning */
-    return 0; /* not implemented */
+UI_METHOD *UI_stunnel() {
+    return UI_OpenSSL();
 }
 #endif
 
